/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

// EXTERNAL INCLUDES
#include <dali-toolkit/dali-toolkit.h>
#include <dali/public-api/dali-wearable.h>
#include <app_common.h>

using namespace Dali;
using namespace Dali::Toolkit;

// This example shows how to implement DALi WatchApplication
class WatchSample : public ConnectionTracker
{
public:
  WatchSample( WatchApplication& application )
  : mApplication( application )
  {
    // Connect to the Application's Init signal
    mApplication.InitSignal().Connect( this, &WatchSample::Create );
  }

  ~WatchSample()
  {
    // Nothing to do here;
  }

  // The Init signal is received once (only) during the Application lifetime
  void Create( Application& application )
  {
    mImageDirectory = app_get_resource_path();

    // Get a handle to the stage
    Stage stage = Stage::GetCurrent();
    CreateClockGUI();

    // Connect tick signal to callback function
    mApplication.TimeTickSignal().Connect( this, &WatchSample::OnTimeTick );
  }

  void CreateClockGUI()
  {
    // Create Root actor.
    Actor mClockStage = Actor::New();
    mClockStage.SetAnchorPoint(AnchorPoint::CENTER);
    mClockStage.SetParentOrigin(ParentOrigin::CENTER);

    // Add clock stage on Dali::Stage.
    Stage::GetCurrent().Add(mClockStage);

    // Create Background.
    mBackground = ImageView::New(mImageDirectory + "bg_01.png");
    mBackground.SetAnchorPoint(AnchorPoint::CENTER);
    mBackground.SetParentOrigin(ParentOrigin::CENTER);
    // Add Background on Stage.
    mClockStage.Add(mBackground);

    // Create Index.
    mIndex = ImageView::New(mImageDirectory + "index_01.png");
    mIndex.SetAnchorPoint(AnchorPoint::CENTER);
    mIndex.SetParentOrigin(ParentOrigin::CENTER);
    // Add Index on Stage.
    mClockStage.Add(mIndex);

    // Create Hour Hand.
    mHourHand = ImageView::New( mImageDirectory + "hand_hour.png");

    // Set Anchor and Parent Property.
    const Vector3 HOUR_HAND_ANCHOR_POINT(0.5f, 180.f/360.f, 0.5f);
    mHourHand.SetAnchorPoint(HOUR_HAND_ANCHOR_POINT);
    mHourHand.SetParentOrigin(ParentOrigin::CENTER);

    // Set Z Position.
    mHourHand.SetZ(0.1f);

    // Add Hour Hand on Background.
    mClockStage.Add(mHourHand);

    // Create Minute Hand.
    mMinuteHand = ImageView::New( mImageDirectory + "hand_min.png");

    // Set Anchor and Parent Property.
    const Vector3 MINUTE_HAND_ANCHOR_POINT(0.5f, 180.f/360.f, 0.5f);
    mMinuteHand.SetAnchorPoint(MINUTE_HAND_ANCHOR_POINT);
    mMinuteHand.SetParentOrigin(ParentOrigin::CENTER);

    // Set Z Position.
    mMinuteHand.SetZ(0.2f);

    // Add Hour Hand on Background.
    mClockStage.Add(mMinuteHand);

    // Create Second Hand.
    mSecondHand = ImageView::New( mImageDirectory + "hand_sec.png");

    // Set Anchor and Parent Property.
    const Vector3 SECOND_HAND_ANCHOR_POINT(0.5f, 160.f/320.f, 0.5f);
    mSecondHand.SetAnchorPoint(SECOND_HAND_ANCHOR_POINT);
    mSecondHand.SetParentOrigin(ParentOrigin::CENTER);

    // Set Z Position.
    mSecondHand.SetZ(0.3f);

    // Add Hour Hand on Background.
    mClockStage.Add(mSecondHand);

    // Locate initial clock hands
    UpdateClockHands(10, 10, 20);
  }

  void UpdateClockHands(int hour, int min, int sec)
  {
    // Update the angle of each hands
    mHourHand.SetOrientation(Degree(hour * 30.f /* 30 degree per hour */ + min * 0.5f /* 0.5 degree per min */), Vector3::ZAXIS);
    mMinuteHand.SetOrientation(Degree(min * 6.f /* 6 degree per min */ + sec * 0.1f /* 0.1 degree per sec */), Vector3::ZAXIS);
    mSecondHand.SetOrientation(Degree(sec * 6.f /* 6 degree per sec */), Vector3::ZAXIS);
  }

  // Update Clock hands on Tick event
  void OnTimeTick( Application& application, const WatchTime& time )
  {
    UpdateClockHands(time.GetHour(), time.GetMinute(), time.GetSecond());
  }

private:
  WatchApplication&  mApplication;
  std::string mImageDirectory;
  TextLabel mTextLabel;

  Actor mClockStage;
  ImageView mBackground;
  ImageView mIndex;
  ImageView mHourHand;
  ImageView mMinuteHand;
  ImageView mSecondHand;
};

// Entry point for Tizen applications
int DALI_EXPORT_API main( int argc, char **argv )
{
  WatchApplication application = WatchApplication::New( &argc, &argv );
  WatchSample test( application );

  application.MainLoop();
  return 0;
}
